#!/bin/bash
#
#	Install script during AppFs installation
#
# VERSION=1
# CHANGES=init

source /tmp/VERSION

FW=${1}
IMAGE_DIR=/tmp/images

### FUNCTIONS ###
function log() {
	prefix=${TYPE:common}
	logfile=/usr/conf/permlog/install_${prefix}-install.log
	pipe=/tmp/install_pipe
	[ -p ${pipe} ] && echo "[install_${prefix}] $(date): ${1}" &>${pipe}
	echo "[install_${prefix}] $(date): ${1}" >> ${logfile}
}

function checkMD5() {
	rootfs_path=${1}
	if [ ! -f ${rootfs_path}/md5sum.list ]; then
		log "ERROR: md5sum.list missing"
		echo 1
	else
		for pkg_name in $(ls ${rootfs_path}/pkg); do
			tmppath=${rootfs_path}/pkg/${pkg_name}
			find ${tmppath} -type f | while read file; do
				filename=$(basename "${file}")
				case ${filename%.*} in
				md5sum|ui-bg_highlight-soft_75_cacaca_1x100)
					## ui-bg_highlight-soft_75_cacaca_1x100 blocks the userapps installation (why ??)
					## we simply avoid the check on this files. must be fixed
					;;
				*)
					MD5=$(md5sum "${file}" | awk '{ print $1; }')
					if [ "$(grep ${MD5} ${rootfs_path}/md5sum.list)" = "" ]; then
						touch /tmp/md5sum.issue
					fi
					;;
				esac
			done
		done

		if [ -f /tmp/md5sum.issue ]; then
			log "ERROR: md5sum check failed"
			rm -f /tmp/md5sum.issue
			echo 1
		fi
		
		echo 0
	fi
}

function installFW() {
	## Cleaning partitions
	log "Cleaning partitions"
	cd /usr/local && ls | grep -v lost | xargs rm -rf
	rm -rf /home/admin/conf
	rm -rf /tmp/conf

	## Extracting firmware
	log "Extracting new Firmware"
	if ! tar xzf ${IMAGE_DIR}/${FW} -C / >/dev/null ; then
		log "ERROR: Extracting new firmware failed"
		echo 1
	else
		## check /usr/local not empty and conf/rootfs exists
		if [ "$(ls /usr/local | wc | awk '{ print $1; }')" = "0" ]; then
			log "ERROR: Extracting files to appfs space failed"
			echo 1
		elif [ ! -d /tmp/conf/rootfs ]; then
			log "ERROR: Extracting files to tmp space failed"
			echo 1
		elif [ ! -d /home/admin/conf/rootfs ]; then
			log "ERROR: Extracting files to userapps space failed"
			echo 1
		else
			echo 0
		fi
	fi
}

function resetUserapps() {
	for file in $(find /home/admin -type f | grep -v "libcrypto\|libssl\|php-cgi"); do
		rm -f ${file}
	done
}

### MAIN ###
rm /usr/conf/permlog/install_${TYPE:common}-install.log

## Remounting AppFs and Userapps space
log "Remounting partitions (rw)."
if ! mount -o remount,rw /usr/local ; then
	log "ERROR: Remounting AppFs-partition (rw) failed"
	touch /tmp/has.issue
	exit 1
fi

if ! mount -o remount,rw /home/admin ; then
	log "ERROR: Remounting UserApps-partition (rw) failed"
	touch /tmp/has.issue
	exit 2
fi

## Installing FW
if [ "$(installFW)" = "1" ]; then
	## userapps must be full. we reset userapps and retry
	if [ "$(resetUserapps)" = "1" ]; then
		touch /tmp/has.issue
		exit 4
	fi
	if [ "$(installFW)" = "1" ]; then
		touch /tmp/has.issue
		exit 5
	fi
fi

## Checking md5 of extracted files on /tmp and /home/admin
if [ "$(checkMD5 "/tmp/conf/rootfs")" = "1" ]; then
	touch /tmp/has.issue
	exit 6
fi
if [ "$(checkMD5 "/home/admin/conf/rootfs")" = "1" ]; then
	## userapps must be full. we reset userapps and retry
	if [ "$(resetUserapps)" = "1" ]; then
		touch /tmp/has.issue
		exit 7
	fi
	if [ "$(installFW)" = "1" ]; then
		touch /tmp/has.issue
		exit 8
	fi
fi

## Checking openssl can be installed
if [[ ! -f /home/admin/lib/libcrypto.1.0.0 ]] || [[ ! -f /home/admin/lib/libssl.1.0.0 ]]; then
	AVAILABLE=$(df | grep mtdblock7 | awk '{ print $4; }')
	REQUIRED=$(du /tmp/conf/rootfs/pkg/openssl/home/admin/lib | awk '{ print $1; }')
	if [ "${REQUIRED}" -gt "$((${AVAILABLE} + 400))" ]; then
		## not enough space. reset userapps and retry
		if [ "$(resetUserapps)" = "1" ]; then
			touch /tmp/has.issue
			exit 9
		fi
		if [ "$(installFW)" = "1" ]; then
			touch /tmp/has.issue
			exit 10
		fi
		AVAILABLE=$(df | grep mtdblock7 | awk '{ print $4; }')
		REQUIRED=$(du | /tmp/conf/rootfs/pkg/openssl/home/admin/lib | awk '{ print $2; }')
		if [ "${REQUIRED}" -gt "$((${AVAILABLE} + 400))" ]; then
			## still not enough space. stop installation process
			touch /tmp/has.issue
			exit 11
		fi
	fi
fi

## Everything goes well. pursuing installation
## moving PKG VERSION.files to /usr/local to be installed
mv /tmp/VERSION /usr/local
mkdir -p /usr/local/conf/rootfs
mv /tmp/conf/rootfs/VERSION.* /usr/local/conf/rootfs
mv /home/admin/conf/rootfs/VERSION.* /usr/local/conf/rootfs

## Setting permissions
log "Setting permissions"
chown -R root:root /usr/local/*
sync; sleep 1; sync

exit 0
